'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.default = createDva;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactRedux = require('react-redux');

var _redux = require('redux');

var _reduxSaga = require('redux-saga');

var _reduxSaga2 = _interopRequireDefault(_reduxSaga);

var _reduxActions = require('redux-actions');

var _effects = require('redux-saga/effects');

var _isPlainObject = require('is-plain-object');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _plugin = require('./plugin');

var _plugin2 = _interopRequireDefault(_plugin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createDva(createOpts) {
  var mobile = createOpts.mobile;
  var initialReducer = createOpts.initialReducer;
  var defaultHistory = createOpts.defaultHistory;
  var routerMiddleware = createOpts.routerMiddleware;
  var setupHistory = createOpts.setupHistory;


  return function dva() {
    var hooks = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var plugin = new _plugin2.default();
    plugin.use(hooks);

    var app = {
      // properties
      _models: [],
      _router: null,
      _store: null,
      _history: null,
      _plugin: plugin,
      // methods
      use: plugin.use.bind(plugin),
      model: model,
      router: router,
      start: start
    };
    return app;

    ////////////////////////////////////
    // Methods

    function model(m) {
      checkModel(m, mobile);
      this._models.push(m);
    }

    // inject model dynamically
    function injectModel(createReducer, onError, m) {
      checkModel(m, mobile);
      var store = this._store;

      // reducers
      store.asyncReducers[m.namespace] = getReducer(m.reducers, m.state);
      store.replaceReducer(createReducer(store.asyncReducers));
      // effects
      if (m.effects) {
        store.runSaga(getSaga(m.effects));
      }
      // subscriptions
      if (m.subscriptions) {
        runSubscriptions(m.subscriptions, this, onError);
      }
    }

    function router(router) {
      _assert2.default.equal(typeof router === 'undefined' ? 'undefined' : (0, _typeof3.default)(router), 'function', 'app.router: router should be function');
      this._router = router;
    }

    function start(container) {
      var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      // support: app.start(opts);
      if ((0, _isPlainObject2.default)(container)) {
        opts = container;
        container = null;
      }

      // support selector
      if (typeof container === 'string') {
        container = document.querySelector(container);
        _assert2.default.ok(container, 'app.start: could not query selector: ' + container);
      }

      _assert2.default.ok(!container || isHTMLElement(container), 'app.start: container should be HTMLElement');
      _assert2.default.ok(this._router, 'app.start: router should be defined');

      // set history
      var history = opts.history || defaultHistory;

      // error wrapper
      var onError = plugin.apply('onError', function (err) {
        throw new Error(err.stack || err);
      });
      var onErrorWrapper = function onErrorWrapper(err) {
        if (err) {
          if (typeof err === 'string') err = new Error(err);
          onError(err);
        }
      };

      // get reducers and sagas from model
      var sagas = [];
      var reducers = (0, _extends3.default)({}, initialReducer);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(this._models), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var m = _step.value;

          reducers[m.namespace] = getReducer(m.reducers, m.state);
          if (m.effects) sagas.push(getSaga(m.effects, onErrorWrapper));
        }

        // extra reducers
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      var extraReducers = plugin.get('extraReducers');
      _assert2.default.ok((0, _keys2.default)(extraReducers).every(function (key) {
        return !(key in reducers);
      }), 'app.start: extraReducers is conflict with other reducers');

      // create store
      var extraMiddlewares = plugin.get('onAction');
      var reducerEnhancer = plugin.get('onReducer');
      var sagaMiddleware = (0, _reduxSaga2.default)();
      var middlewares = [sagaMiddleware].concat((0, _toConsumableArray3.default)(extraMiddlewares));
      if (routerMiddleware) {
        middlewares = [routerMiddleware(history)].concat((0, _toConsumableArray3.default)(middlewares));
      }
      var devtools = window.devToolsExtension || function () {
        return function (noop) {
          return noop;
        };
      };
      var enhancers = [_redux.applyMiddleware.apply(undefined, (0, _toConsumableArray3.default)(middlewares)), devtools()];
      var store = this._store = (0, _redux.createStore)(createReducer(), opts.initialState || {}, _redux.compose.apply(undefined, enhancers));

      function createReducer(asyncReducers) {
        return reducerEnhancer((0, _redux.combineReducers)((0, _extends3.default)({}, reducers, extraReducers, asyncReducers)));
      }

      // extend store
      store.runSaga = sagaMiddleware.run;
      store.asyncReducers = {};

      // store change
      var listeners = plugin.get('onStateChange');
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator3.default)(listeners), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var listener = _step2.value;

          store.subscribe(listener);
        }

        // start saga
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      sagas.forEach(sagaMiddleware.run);

      // setup history
      if (setupHistory) setupHistory.call(this, history);

      // run subscriptions
      var subs = this._models.reduce(function (ret, _ref) {
        var subscriptions = _ref.subscriptions;

        return [].concat((0, _toConsumableArray3.default)(ret), (0, _toConsumableArray3.default)(subscriptions || []));
      }, []);
      runSubscriptions(subs, this, onErrorWrapper);

      // inject model after start
      this.model = injectModel.bind(this, createReducer, onErrorWrapper);

      // If has container, render; else, return react component
      if (container) {
        render(container, store, this, this._router);
        plugin.apply('onHmr')(render.bind(this, container, store, this));
      } else {
        return getProvider(store, this, this._router);
      }
    }

    ////////////////////////////////////
    // Helpers

    function getProvider(store, app, router) {
      return function () {
        return _react2.default.createElement(
          _reactRedux.Provider,
          { store: store },
          router({ app: app, history: app._history })
        );
      };
    }

    function render(container, store, app, router) {
      var ReactDOM = require('react-dom');
      ReactDOM.render(_react2.default.createElement(getProvider(store, app, router)), container);
    }

    function checkModel(model, mobile) {
      _assert2.default.ok(model.namespace, 'app.model: namespace should be defined');
      _assert2.default.ok(mobile || model.namespace !== 'routing', 'app.model: namespace should not be routing, it\'s used by react-redux-router');
      _assert2.default.ok(!model.subscriptions || Array.isArray(model.subscriptions), 'app.model: subscriptions should be Array');
      _assert2.default.ok(!model.reducers || (0, _typeof3.default)(model.reducers) === 'object' || Array.isArray(model.reducers), 'app.model: reducers should be Object or array');
      _assert2.default.ok(!Array.isArray(model.reducers) || (0, _typeof3.default)(model.reducers[0]) === 'object' && typeof model.reducers[1] === 'function', 'app.model: reducers with array should be app.model({ reducers: [object, function] })');
      _assert2.default.ok(!model.effects || (0, _typeof3.default)(model.effects) === 'object', 'app.model: effects should be Object');
    }

    function isHTMLElement(node) {
      return (typeof node === 'undefined' ? 'undefined' : (0, _typeof3.default)(node)) === 'object' && node !== null && node.nodeType && node.nodeName;
    }

    function getReducer(reducers, state) {
      if (Array.isArray(reducers)) {
        return reducers[1]((0, _reduxActions.handleActions)(reducers[0], state));
      } else {
        return (0, _reduxActions.handleActions)(reducers || {}, state);
      }
    }

    function getSaga(effects, onError) {
      return _regenerator2.default.mark(function _callee() {
        var key, watcher;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.t0 = _regenerator2.default.keys(effects);

              case 1:
                if ((_context.t1 = _context.t0()).done) {
                  _context.next = 8;
                  break;
                }

                key = _context.t1.value;
                watcher = getWatcher(key, effects[key], onError);
                _context.next = 6;
                return (0, _effects.fork)(watcher);

              case 6:
                _context.next = 1;
                break;

              case 8:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      });
    }

    function getWatcher(key, _effect, onError) {
      var _marked = [sagaWithCatch].map(_regenerator2.default.mark);

      var effect = _effect;
      var type = 'takeEvery';
      if (Array.isArray(_effect)) {
        effect = _effect[0];
        var opts = _effect[1];
        if (opts && opts.type) {
          type = opts.type;
        }
        _assert2.default.ok(['watcher', 'takeEvery', 'takeLatest'].indexOf(type) > -1, 'app.start: effect type should be takeEvery, takeLatest or watcher');
      }

      function sagaWithCatch() {
        var _args2 = arguments;
        return _regenerator2.default.wrap(function sagaWithCatch$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.prev = 0;
                _context2.next = 3;
                return effect.apply(undefined, _args2);

              case 3:
                _context2.next = 8;
                break;

              case 5:
                _context2.prev = 5;
                _context2.t0 = _context2['catch'](0);

                onError(_context2.t0);

              case 8:
              case 'end':
                return _context2.stop();
            }
          }
        }, _marked[0], this, [[0, 5]]);
      }

      switch (type) {
        case 'watcher':
          return sagaWithCatch;
        case 'takeEvery':
          return _regenerator2.default.mark(function _callee2() {
            return _regenerator2.default.wrap(function _callee2$(_context3) {
              while (1) {
                switch (_context3.prev = _context3.next) {
                  case 0:
                    _context3.next = 2;
                    return (0, _reduxSaga.takeEvery)(key, sagaWithCatch);

                  case 2:
                  case 'end':
                    return _context3.stop();
                }
              }
            }, _callee2, this);
          });
        case 'takeLatest':
          return _regenerator2.default.mark(function _callee3() {
            return _regenerator2.default.wrap(function _callee3$(_context4) {
              while (1) {
                switch (_context4.prev = _context4.next) {
                  case 0:
                    _context4.next = 2;
                    return (0, _reduxSaga.takeLatest)(key, sagaWithCatch);

                  case 2:
                  case 'end':
                    return _context4.stop();
                }
              }
            }, _callee3, this);
          });
        default:
          throw new Error('app.start: unsupport effect type ' + type);
      }
    }

    function runSubscriptions(subs, app, onError) {
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = (0, _getIterator3.default)(subs), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var sub = _step3.value;

          _assert2.default.ok(typeof sub === 'function', 'app.start: subscription should be function');
          sub({ dispatch: app._store.dispatch, history: app._history }, onError);
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }
    }
  };
}
module.exports = exports['default'];